@@ -945,14 +945,9 @@ render_rect_alpha (EelCanvasRect *rect,
 		memcpy (data + i*rowstride, data, width*4);
 	}
 	
-	gdk_pixbuf_render_to_drawable_alpha (pixbuf,
-					     drawable,
-					     0, 0,
-					     x, y,
-					     width, height,
-					     GDK_PIXBUF_ALPHA_FULL,
-					     255,
-					     GDK_RGB_DITHER_NONE, 0, 0);
+	gdk_draw_pixbuf (drawable, NULL, pixbuf,
+			 0, 0, x, y, width, height,
+			 GDK_RGB_DITHER_NONE, 0, 0);
 	g_object_unref (pixbuf);
 }
 
