@@ -43,6 +43,8 @@
 struct EelImageChooserDetails {
 	GtkListStore *list_store;
 
+	GtkTreeRowReference *selected_row;
+
 	int maximum_height;
 };
 
@@ -68,9 +70,21 @@ GNOME_CLASS_BOILERPLATE (EelImageChooser, eel_image_chooser,
 			 GtkTreeView, GTK_TYPE_TREE_VIEW)
 
 static void
-eel_image_chooser_selection_changed (GtkTreeSelection *selection, EelImageChooser *image_chooser)
+eel_image_chooser_row_activated (GtkTreeSelection *selection,
+				 GtkTreePath *path,
+				 GtkTreeViewColumn *column,
+				 EelImageChooser *image_chooser)
 {
-	g_signal_emit (image_chooser, image_chooser_signals[SELECTION_CHANGED], 0);
+	if (image_chooser->details->selected_row) {
+		gtk_tree_row_reference_free (image_chooser->details->selected_row);
+	}
+	
+	image_chooser->details->selected_row = 
+		gtk_tree_row_reference_new (GTK_TREE_MODEL (image_chooser->details->list_store), path);
+
+	g_signal_emit (image_chooser, 
+		       image_chooser_signals[SELECTION_CHANGED], 
+		       0);
 }
 
 static void
@@ -105,6 +119,10 @@ eel_image_chooser_finalize (GObject *object)
 	
 	image_chooser = EEL_IMAGE_CHOOSER (object);
 
+	if (image_chooser->details->selected_row) {
+		gtk_tree_row_reference_free (image_chooser->details->selected_row);
+	}
+
 	g_object_unref (image_chooser->details->list_store);
 
 	g_free (image_chooser->details);
@@ -194,9 +212,13 @@ eel_image_chooser_instance_init (EelImageChooser *image_chooser)
 	
 	gtk_tree_view_append_column (GTK_TREE_VIEW (image_chooser), column);
 
-	/* Set up selection handler */
-	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (image_chooser)), "changed",
-			  G_CALLBACK (eel_image_chooser_selection_changed), image_chooser);
+	eel_gtk_tree_view_set_activate_on_single_click 
+		(GTK_TREE_VIEW (image_chooser), TRUE);
+
+	g_signal_connect (GTK_TREE_VIEW (image_chooser), 
+			  "row_activated",
+			  G_CALLBACK (eel_image_chooser_row_activated),
+			  image_chooser);
 }
 
 GtkWidget *
@@ -236,21 +258,11 @@ eel_image_chooser_insert_row (EelImageChooser *image_chooser,
 static GtkTreePath *
 eel_image_chooser_get_selected_path (const EelImageChooser *image_chooser)
 {
-	GtkTreeIter iter;
-	GtkTreePath *path;
-	gboolean selected;
-	
-	selected = gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (image_chooser)),
-						    NULL, &iter);
-
-	if (selected == FALSE) {
+	if (image_chooser->details->selected_row) {
+		return gtk_tree_row_reference_get_path (image_chooser->details->selected_row);
+	} else {
 		return NULL;
 	}
-
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (image_chooser->details->list_store),
-					&iter);
-
-	return path;
 }
 
 int
@@ -298,6 +310,11 @@ eel_image_chooser_set_selected_row (EelImageChooser *image_chooser,
 {
 	GtkTreePath *path;
 
+	if (image_chooser->details->selected_row) {
+		gtk_tree_row_reference_free (image_chooser->details->selected_row);
+		image_chooser->details->selected_row = NULL;
+	}
+
 	if (icon_position < 0) {
 		gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (image_chooser)));
 		return;
@@ -306,6 +323,11 @@ eel_image_chooser_set_selected_row (EelImageChooser *image_chooser,
 	path = gtk_tree_path_new ();
 	gtk_tree_path_append_index (path, icon_position);
 	gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW (image_chooser)), path);
+
+	image_chooser->details->selected_row = gtk_tree_row_reference_new
+		(gtk_tree_view_get_model (GTK_TREE_VIEW (image_chooser)),
+		 path);
+
 	gtk_tree_path_free (path);
 }
 
@@ -358,7 +380,9 @@ eel_scrolled_image_chooser_new (GtkWidget **image_chooser_out)
  	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
  					GTK_POLICY_NEVER,
  					GTK_POLICY_AUTOMATIC);
-
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+					     GTK_SHADOW_IN);
+	
 	image_chooser = eel_image_chooser_new ();
 	gtk_widget_show (image_chooser);
 	
